%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2013 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function disconnectSimulinkModelFromDM(callbackInfo)
	postSaveFcn = get_param(gcs, 'PostSaveFcn');
	if strcmp(postSaveFcn, 'saveSimulinkModelToDM()') == 0
		disp('Model is not connected to DM, no disconnection needed');
		return;
	end
	
	%open a SaveAs dialog to get the path from the user
	try
		evalin('base', 'createDMConnection');
		dmConn = evalin('base', 'dmConnection');
	catch
		return;
	end	
	
	if isjava(dmConn) == 0
		return;
	end	
	
	mdlName = get_param(gcs,'name');
	ext = 'mdl';
	try
		ext = get_param(0, 'ModelFileFormat');
	catch
	end
	mdlFileName = strcat(mdlName,'.',ext);
	
	fullFileName = char(dmConn.openSaveAsDialogForSimulinkModel(mdlFileName));
	if isempty(fullFileName) == 0
		%clear the callbacks
        
        % unlock library before save
        libraryLocked = get_param(gcs, 'Lock');
        if strcmp(libraryLocked, 'on') == 1
            set_param(gcs, 'Lock', 'off');
        end 
		
        set_param(gcs,'PostLoadFcn','');
		set_param(gcs,'PostSaveFcn','');
		set_param(gcs,'CloseFcn','');
	
		%save the model in the new location
		save_system(gcs, fullFileName);
		
        % restore the library lock
        if strcmp(libraryLocked, 'on') == 1
            set_param(gcs, 'Lock', 'on');
        end 
        
        simMdlPath = char(dmConn.getSimModelPathDir());
        if isempty(simMdlPath) == 0
            disp('Changing location to disconnected model');
            cd(simMdlPath);
        end
        
		disp('Disconnection done.');
	end
end