function [ minXY ] = getMinXYOfSimulinkModel( sys )
    minXY = [];
    if isempty(sys) == 1
        return;
    end    

    minX = 0;
    minY = 0;
    try
        % first, traverse the model's blocks, and find the minXY position
        blks = get_param(sys, 'Blocks');
        sysName = get_param(sys, 'Name');
        numOfBlocks = size(blks);
        for x = 1:numOfBlocks
            blockName = blks{x};
            tempBlockName = regexprep(blockName, '/', '//');
            fullBlockName = strcat(sysName, '/', tempBlockName);
            blockPosition = getBlockPositionWithText(sys, blockName, fullBlockName);
            if minX == 0 || blockPosition(1) < minX
                minX = blockPosition(1);
            end
            if minY == 0 || blockPosition(2) < minY
                minY = blockPosition(2);
            end
        end 
    
        % next, traverse the model's lines, and find the minXY position
        lines = get_param(sys, 'Lines');
        numOfLines = size(lines);
        for x = 1:numOfLines
            line = lines(x);
            tmpMinXY = getMinXYOfLine(line);
            if isempty(tmpMinXY) == 0
                if(tmpMinXY(1) < minX)
                    minX = tmpMinXY(1);
                end
                if(tmpMinXY(2) < minY)
                    minY = tmpMinXY(2);
                end
            end              
        end

        % last, traverse the model's annotation, and find the minXY position
        annotationHandles = find_system(sys, 'FindAll', 'on', 'type', 'annotation');
        numOfAnnotations = size(annotationHandles);
        for x = 1:numOfAnnotations
            annotationPosition = get_param (annotationHandles(x),'Position');
            if minX == 0 || annotationPosition(1) < minX
                minX = annotationPosition(1);
            end
            if minY == 0 || annotationPosition(2) < minY
                minY = annotationPosition(2);
            end
        end
    catch e
        minX = 0;
        minY = 0;
    end
    
    minXY = [minX, minY];    
end


function [minXY] = getMinXYOfLine(line)
    minXY = [];
    minX = 0;
    minY = 0;
    if isempty(line) == 0
        linePoints = line.Points;
        ptsSize = size(linePoints);
        for i = 1 : ptsSize
            if minX == 0 || linePoints(i) < minX
                minX = linePoints(i, 1);
            end
            if minY == 0 || linePoints(i+1) < minY
                minY = linePoints(i, 2);
            end 
        end
        
        if isempty(line.Branch) == 0
            branchSize = size(line.Branch);
            for j = 1 : branchSize
                tmpMinXY = getMinXYOfLine(line.Branch(j));
                if isempty(tmpMinXY) == 0
                    if(tmpMinXY(1) < minX)
                        minX = tmpMinXY(1);
                    end
                    if(tmpMinXY(2) < minY)
                        minY = tmpMinXY(2);
                    end
                end 
            end    
        end
        minXY = [minX, minY];
    end    
end

function [textSize] = getTextSize(sys, blockName, fullBlockName)
    textSize = [];
    if isempty(sys) == 0 && isempty(blockName) == 0 && isempty(fullBlockName) == 0
        showName = get_param(fullBlockName, 'ShowName');
        if isempty(showName) == 1 || strcmp(showName, 'off')
            return;
        end
        
                
        fontName = get_param(fullBlockName, 'FontName');
        if isempty(fontName) == 1 || strcmp(fontName, 'auto') == 1
            fontName = get_param(sys, 'DefaultBlockFontName'); 
        end 
        
        fontSize = get_param(fullBlockName, 'FontSize');
        if isempty(fontSize) == 1 || fontSize == -1
            fontSize = get_param(sys, 'DefaultBlockFontSize'); 
        end 
        
        largestLine = [];
        numOfLines = 0;
        if isempty(strfind(blockName, char(10))) == 0
            remain = blockName;
            while true
                [str, remain] = strtok(remain, char(10));
                if isempty(str) == 1
                    break;
                end
                numOfLines = numOfLines + 1;
                if length(char(str)) > length(char(largestLine))
                    largestLine = str;
                end    
            end
        else
            largestLine = blockName;
            numOfLines = 1;
        end
        
        textWidth = 0;
        textHeight = 0;
        if isempty(fontName) == 0 && isempty(fontSize) == 0
            try
                font = java.awt.Font(fontName, 0, fontSize);
                img = java.awt.image.BufferedImage(1, 1, 2);
                fm = img.getGraphics().getFontMetrics(font);
                textWidth = fm.stringWidth(largestLine);
                textHeight = fm.getHeight() * numOfLines;
            catch
            end    
        end 
        textSize = [textWidth, textHeight];
    end    
end

function [position] = getBlockPositionWithText(sys, blockName, fullBlockName)
    position = [];
    if isempty(sys) == 0 && isempty(blockName) == 0 && isempty(fullBlockName) == 0
        position = get_param(fullBlockName, 'Position');
        textSize = getTextSize(sys, blockName, fullBlockName);
        if isempty(textSize) == 0
            textWidth = textSize(1);
            textHeight = textSize(2);
            
            blkWidth = position(3) - position(1);
            blkHeight = position(4) - position(2);
            
            % 0 - no need to calculate
            % 1 - add only margin
            % 2 - add full text size
            bCalcX = 0;
            bCalcY = 0;
            blockRotation = get_param(fullBlockName, 'BlockRotation');
            namePlacement = get_param(fullBlockName, 'NamePlacement');
            if blockRotation == 0 && strcmp(namePlacement,'normal') == 1 
                bCalcX = 1;
                bCalcY = 0;
            elseif blockRotation == 0 && strcmp(namePlacement,'alternate') == 1
                bCalcX = 1;
                bCalcY = 2;    
            elseif blockRotation == 270 && strcmp(namePlacement,'normal') == 1
                bCalcX = 0;
                bCalcY = 1;    
            elseif blockRotation == 270 && strcmp(namePlacement,'alternate') == 1
                bCalcX = 2;
                bCalcY = 1;        
            end 
            
            if bCalcX == 1 && textWidth > blkWidth
                position(1) = position(1) - round((textWidth - blkWidth)/2);
            elseif bCalcX == 2
                position(1) = position(1) - textWidth;    
            end    
            
            if bCalcY == 1 && textHeight > blkHeight
                position(2) = position(2) - round((textHeight - blkHeight)/2);
            elseif bCalcY == 2
                position(2) = position(2) - textHeight;    
            end    
        end    
    end
end
