%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2013 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function loadReferenceModelsFromDM( sys, bRecursive )

    try
		evalin('base', 'createDMConnection');
		dmConn = evalin('base', 'dmConnection');
	catch
		return;
	end	
	
	if isjava(dmConn) == 0
		fprintf('ERROR: java object (DMConnection) allocation failed\n');
    else
        if isobject(sys) == 0
            internalSysName = get_param(sys,'name');
            sysName = regexprep(internalSysName, '\n', ' ');
            blocks = get_param (sys,'Blocks');
            numOfBlocks = size(blocks);

            for x = 1:numOfBlocks
                tempBlockName = blocks{x};
                internalBlockName = regexprep(tempBlockName, '/', '//');
                legalBlockName = regexprep(internalBlockName, '\n', ' ');
                internalBlockPath = horzcat(sysName, '/',legalBlockName);
                
                blockType = get_param (internalBlockPath,'BlockType');
                if strcmp(blockType, 'ModelReference') == 1
                    refMdlName = get_param(internalBlockPath, 'ModelName');
                    refSysName = regexprep(refMdlName, '\n', ' ');
                    downloadedSimMdlDir = char(dmConn.downloadSimulinkModelNoGUI(refSysName));
                    if isempty(downloadedSimMdlDir) == 0
                        if strcmp(downloadedSimMdlDir, 'cancel') == 1
                            return;
                        end	
			
                        addpath(downloadedSimMdlDir);
                        msg = strcat('Loading reference model "', refSysName, '"...'); 
                        disp(msg);
                        refSys = load_system(refSysName);
                        
                        if bRecursive == 1
                            loadReferenceModelsFromDM(refSys, bRecursive);
                        end
                        
                        try
                            set_param(refSys,'PostLoadFcn','postOpenModelActions()');
                            set_param(refSys,'PostSaveFcn','');
                            set_param(refSys,'CloseFcn','');
                            save_system(refSys, [], 'OverWriteIfChangedOnDisk', true);
                        catch e
                            msg = strcat('Failed to save model, model "', refSysName, '" wont be connected to DM');
                            disp(msg);
                        end    
                        h = get_param(sys,'object');
                        h.refreshModelBlocks;
                        
                        msg = strcat('Loading reference model "', refSysName, '" done'); 
                        disp(msg);
                        
                           
                        close_system(refSys, 1);
                    else
                        disp('Failed to get model from DM');
                    end
                end
            end
        end
    end
end

