%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2013 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function bSuccess = moveSimulinkModelToDM()
	try
		evalin('base', 'createDMConnection');
		dmConn = evalin('base', 'dmConnection');
	catch
		return;
	end	
	
	if isjava(dmConn) == 0
		return;
	end	

	sysName = get_param(gcs,'name');
	moveToAction = char(dmConn.openMoveToDMDialog(sysName));
	if isempty(moveToAction) == 1
		return;
    end	
	
    projectArea = char(dmConn.getProjectArea());
    workspace = char(dmConn.getWorkspace());
	msg = sprintf('Moving "%s" to DM (ProjectArea: %s, Workspace: %s)...',sysName, projectArea, workspace); 
	disp(msg);
	
    withRefModels = char(dmConn.getWithReferenceModels());
	bSuccess = uploadSimulinkModelToDM(sysName, withRefModels, moveToAction, gcs);
	if bSuccess == 1
        try 
            % unlock library before save
            libraryLocked = get_param(sysName, 'Lock');
            if strcmp(libraryLocked, 'on') == 1
                set_param(sysName, 'Lock', 'off');
            end  
            
            set_param(sysName,'PostSaveFcn','postOpenFromDMActions()');
            save_system(sysName);
            
            msg = sprintf('Moving "%s" to DM done',sysName); 
            disp(msg);
		
            %lock the model since save/moveto release the locking
            if strcmp(moveToAction, 'MoveToAndLock') == 1
                dmConn.lockSimulinkModel(sysName);
            end	
            
            % restore the library lock
            if strcmp(libraryLocked, 'on') == 1
                set_param(sysName, 'Lock', 'on');
            end  
        catch
            msg = strcat('Failed to save model, model "', sysName, '" wont be connected to DM');
            disp(msg);
        end
	else
		disp('Failed to move model to DM');
	end		
end