%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2013 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function bSuccess = openSimulinkModelFromDM()
	bSuccess = 0;
	try
		evalin('base', 'createDMConnection');
		dmConn = evalin('base', 'dmConnection');
	catch
		return;
	end	
	
	if isjava(dmConn) == 0
		fprintf('ERROR: java object (DMConnection) allocation failed\n');
	else
		downloadedSimMdl = char(dmConn.downloadSimulinkModel());
		if isempty(downloadedSimMdl) == 0
			if strcmp(downloadedSimMdl, 'cancel') == 1
				return;
            end	
			
            downloadedSimMdlDir = char(dmConn.getWorkingDirectory());
            if isempty(downloadedSimMdlDir) == 1
                return;
            end
            
            disp('Changing to working folder (can be set via "Settings" button)');
            addpath(downloadedSimMdlDir);
            cd(downloadedSimMdlDir);
            
			disp('Opening model...');
            warning('off','Simulink:modelReference:modelNotFound');
			sys = load_system(downloadedSimMdl);
            
            libraryLocked = get_param(sys, 'Lock');
            sysName = get_param(sys, 'Name');
            
            try
                % unlock library before save
                if strcmp(libraryLocked, 'on') == 1
                    set_param(sys, 'Lock', 'off');
                end 
                
                set_param(sys,'PostLoadFcn','postOpenFromDMActions()');
                set_param(sys,'PostSaveFcn','');
                set_param(sys,'CloseFcn','');
                save_system(sys, [], 'OverWriteIfChangedOnDisk', true);
                close_system(sys);
                sys = load_system(downloadedSimMdl);
                
                % restore the library lock
                if strcmp(libraryLocked, 'on') == 1
                    set_param(sys, 'Lock', 'on');
                end  
            catch
                msg = strcat('Failed to save model, model "', sysName, '" wont be connected to DM');
                disp(msg);
            end
            
            open_system(downloadedSimMdl);
            
            withRefModels = char(dmConn.getWithReferenceModels());
            if strcmp(withRefModels, 'include') == 1
                loadReferenceModelsFromDM(sys, 0);
            elseif strcmp(withRefModels, 'includeRecursive') == 1
                loadReferenceModelsFromDM(sys, 1);
            end
            
            % remove the dirty flag (model might be modified when loading
            % refererence models)
            if strcmp(libraryLocked, 'off') == 1
                set_param(sys, 'dirty', 'off');
            end    
            
            warning('on','Simulink:modelReference:modelNotFound');
			
			disp('Opening model done.');
			
			bSuccess = 1;
		else
			disp('Failed to get model from DM');
		end
	end		
end

