%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM. Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%%  Copyright IBM Corporation 2013 
%% U.S. Government Users Restricted Rights: Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function sl_customization(cm)

  %% Register custom menu function.
  cm.addCustomMenuFcn('Simulink:ToolsMenu', @getMyMenuItems);
end

%% Define the custom menu function.
function schemaFcns = getMyMenuItems(callbackInfo) 
  schemaFcns = {@menuDM,...
                }; 
end

%% Define the schema function for group menu item.
function schema = menuDM(callbackInfo)
  schema = sl_container_schema;
  schema.label = 'Design Manager';
  schema.childrenFcns = {@menuPublish, @menuNavigateToServer, 'separator',@menuMoveToDM, @menuLockModel, @menuUnlockModel, @menuDisconnectModel, 'separator', @menuReferenceModels};
end

function schema = menuPublish(callbackInfo)
  schema = sl_action_schema;
  schema.label = 'Publish model...';
  schema.userdata = 'publish model to RDM server';	
  schema.callback = @publishToServer;
end 

function publishToServer(callbackInfo)
    publishSimulinkModelToDM();
end

function schema = menuNavigateToServer(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Navigate to Design Manager...';
	schema.userdata = 'navigate to server';	
	schema.callback = @navigateToServer;
end 

function navigateToServer(callbackInfo)
    navigateToDMServer(gcs);
end

function schema = menuMoveToDM(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Move to Design Manager...';
	schema.userdata = 'Move to Design Manager...';	
	schema.callback = @moveToDM;
end

function moveToDM(callbackInfo)
	moveSimulinkModelToDM();
end

function schema = menuSaveOnDM(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Save to server';
	schema.userdata = 'save to server';	
	schema.callback = @saveToDM;
end

function saveToDM(callbackInfo)
	saveSimulinkModelToDM(0);
end

function schema = menuLockModel(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Lock Model';
	schema.userdata = 'Lock Model';	
	schema.callback = @lockModelInDM;
end

function lockModelInDM(callbackInfo)
	lockSimulinkModelInDM();
end

function schema = menuUnlockModel(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Unlock Model';
	schema.userdata = 'Unlock Model';	
	schema.callback = @unlockModelInDM;
end

function unlockModelInDM(callbackInfo)
	unlockSimulinkModelInDM(0);
end

function schema = menuDisconnectModel(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Disconnect Model...';
	schema.userdata = 'Disconnect Model...';	
	schema.callback = @disconnectModelFromDM;
end

function disconnectModelFromDM(callbackInfo)
  	disconnectSimulinkModelFromDM();
end

function schema = menuReferenceModels(callbackInfo)
  schema = sl_container_schema;
  schema.label = 'Reference Models';
  schema.childrenFcns = {@menuLoadRefModels, @menuLoadRefModelsRecursive};
end

function schema = menuLoadRefModels(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Load From DM';
	schema.userdata = 'Load From DM';	
	schema.callback = @OpenRefModels;
end

function OpenRefModels(callbackInfo)
    sysHandle = get_param(bdroot, 'handle');
	loadReferenceModelsFromDM(sysHandle, 0);
end

function schema = menuLoadRefModelsRecursive(callbackInfo)
	schema = sl_action_schema;
	schema.label = 'Load From DM Recursive';
	schema.userdata = 'Load From DM Recursive';	
	schema.callback = @OpenRefModelsRecursive;
end

function OpenRefModelsRecursive(callbackInfo)
    sysHandle = get_param(bdroot, 'handle');
	loadReferenceModelsFromDM(sysHandle, 1);
end