%% Licensed Materials - Property of IBM
%% Restricted Materials of IBM.  Reproduction, modification and redistribution are prohibited.
%% IBM Rational Rhapsody Design Manager
%% © Copyright IBM Corporation 2013  
%% U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.

function [bSuccess] = uploadSimulinkModelToDM(sysName, withRefModels, moveToAction, origSystem)
	path2mdl = which(sysName);
    if isempty(path2mdl)
        msg = strcat('Could not find path to model "', sysName, '"');
        disp(msg);
        bSuccess = false;
        return;
    end    
	pubDir = strcat(tempdir, 'SimulinkImporterData');
	[s,mess,messid] = mkdir(pubDir);

	bSuccess = 0;
  	sys = load_system(path2mdl);
	if isobject(sys) == 0
		tempSysName = get_param(sys,'name');
		
		sysName = tempSysName;
		pubDir = strcat(pubDir, '/',sysName);
		[s,mess,messid] = mkdir(pubDir);

		% Prepare java objects:
		publisher = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.Publisher', pubDir);
		if isjava(publisher) == 0 
			fprintf('ERROR: java object (Publisher) allocation failed\n');
			return;
		end
    
		helper = javaObject('com.ibm.rational.rdm.simulinkintegration.publish.MscriptHelper');
		if isjava(helper) == 0 
			fprintf('ERROR: java object (MscriptHelper) allocation failed\n');
			return;
        end
        
        if strcmp(moveToAction, 'save') == 1
            publisher.setInSaveModel(1);
        end   
        
		publisher.setModelFile(path2mdl);
		publisher.createModel(sysName);
     
        try
            getSystemData(sys, publisher, '', helper, pubDir, path2mdl, withRefModels, moveToAction, origSystem);
        catch
            return;
        end
        
		% Serialize and move to DM
    	publisher.Serialize();   
        bSuccess = publisher.moveToDM();    
    end
end

function getSystemData(sys, publisher, parentSysPath, helper, publishDir, path2mdl, withRefModels, moveToAction, origSystem)
    try
        evalin('base', 'createDMConnection');
        dmConn = evalin('base', 'dmConnection');
    catch
        return;
    end	
    if isjava(dmConn) == 0
        return;
    end    
	    
    internalSysName = get_param(sys,'name');
    sysName = regexprep(internalSysName, '\n', ' ');
    sysDescript = get_param(sys,'Description');
    
    sysUriName = char(helper.legalizeMdlElemName(sysName));
    
    subSystem = 0;

    isLibrary = 0;
    try
        libraryType = get_param(sys, 'LibraryType');
        if strcmp(libraryType, 'BlockLibrary') == 1
            isLibrary = 1;
        end
    catch
    end
    
        
    if ~isempty(parentSysPath)
        simSystem = publisher.addSystem(sysName, sysUriName, sysDescript, 0, isLibrary);
        subSystem = 1;
    else
        simSystem = publisher.addSystem(sysName, sysUriName, sysDescript, 1, isLibrary);
    end
    
    legalParentSysPath = regexprep(parentSysPath, '\n', ' ');
    
    sysPath = sysName;
    if isempty(legalParentSysPath) == 0
        sysPath = strcat(legalParentSysPath,'/',sysName);
    end
        
	% copy the model file in the publishDir
    if subSystem == 0
        copyfile(path2mdl, publishDir);
    end
    
    % copy the 'slblocks.m' is case we are a library
    if isLibrary == 1
        slblocksLocation = char(dmConn.getLibrarySlblocksPath());
        if isempty(slblocksLocation) == 0
            copyfile(slblocksLocation, publishDir);
        end    
    end    
      
    minXY = getMinXYOfSimulinkModel(sys);
    if isempty(minXY) == 0
        simSystem.setMinXY(minXY(1), minXY(2));
    end    
    
    % Print diagram:
    % ======================
    stateFlowType = '';
    try
        stateFlowType = get_param(sysPath, 'SFBlockType');
    catch
  %      fprintf('No SFBlockType found for %s\n', sysPath);
    end
    
    if strcmp(stateFlowType, 'Chart') == 1
        printChartDiagram(sys, sysPath, publishDir, helper, origSystem, simSystem);
    elseif isempty(stateFlowType) == 1 || strcmp(stateFlowType, 'NONE') == 1
        printSystemDiagram(sys, publishDir, helper, origSystem, simSystem);
    end
    
    % Get system lines:
    % ======================
    SysLines = get_param (sys,'Lines');
    numOfLines = size(SysLines);
    
    for z = 1:numOfLines
        lineName = SysLines(z).Name;
        srcBlock = get_param(SysLines(z).SrcBlock, 'Name');
        legalSrcBlock = char(helper.legalizeMdlElemName(srcBlock));
        destBlock = get_param(SysLines(z).DstBlock, 'Name');
        legalDestBlock = char(helper.legalizeMdlElemName(destBlock));
        
        publisher.addSignal(lineName, legalSrcBlock, legalDestBlock);
    end
    
    % Get system blocks:
    % ======================
    blocks = get_param (sys,'Blocks');
    numOfBlocks = size(blocks);

    for x = 1:numOfBlocks
        
        tempBlockName = blocks{x};
        internalBlockName = regexprep(tempBlockName, '/', '//');
        legalBlockName = regexprep(internalBlockName, '\n', ' ');
        tempBlockPath = strcat(sysPath,'/',legalBlockName);
        internalBlockPath = horzcat(sysPath, '/',legalBlockName);
        blockPath = regexprep(tempBlockPath, '\n', ' ');
        
        if subSystem == 1
            legalSysPath = char(helper.legalizeMdlElemName(sysPath));
            legalBlockName2 = char(helper.legalizeMdlElemName(legalBlockName));
            uriName = strcat(legalSysPath, '_subsys_', legalBlockName2);
        else
            uriName = char(helper.legalizeMdlElemName(blockPath));
        end
        
        blockPosition = get_param (internalBlockPath,'Position');
        blockType = get_param (internalBlockPath,'BlockType');
       
        referenceBlock = get_param(internalBlockPath, 'ReferenceBlock'); 
        if isempty(referenceBlock) == 0
            blockType = strrep(referenceBlock, '/', '_');
        end
                
        blockDesc = get_param (internalBlockPath,'Description');
        
        dblArray = javaArray ('java.lang.Double', 4);

        dblArray(1) = java.lang.Double(blockPosition(1));
        dblArray(2) = java.lang.Double(blockPosition(2));
        dblArray(3) = java.lang.Double(blockPosition(3));
        dblArray(4) = java.lang.Double(blockPosition(4));
        
        simBlock = publisher.addBlock(sysUriName, legalBlockName, uriName, blockType, blockDesc, dblArray);
        
        if isempty(referenceBlock) == 0
            refBlockName = referenceBlock;
            refBlockName = regexprep(refBlockName, '\n', ' ');
            refBlockUriName = char(helper.legalizeMdlElemName(refBlockName));
            publisher.addReferenceBlock(sysUriName, refBlockName, refBlockUriName);
        end
        
        ObjParamsStruct = get_param(internalBlockPath, 'DialogParameters');
        numOfParams = 0;
        try
            if isempty(ObjParamsStruct) == 0
                ObjParams = struct2cell(ObjParamsStruct);
                InternalNames = fieldnames(ObjParamsStruct);
                numOfParams = size(ObjParams);
            end    
        catch
            fprintf('ERROR: struct2cell() failure\n');
        end
         
        getProps = 1;
        if strcmp(blockType, 'Scope') == 1
            getProps = 0;
        elseif numOfParams < 1
            getProps = 0;
        end
        
        % Get block parameters:
        % ======================
        if getProps > 0
            for y = 1:numOfParams                
                paramName = ObjParams{y}.Prompt;                
                paramVal = get_param(internalBlockPath, InternalNames{y});
                if ischar(paramName) && ischar(paramVal)
                    publisher.addProperty(paramName, paramVal);
                end
            end
        end
        
        % get sub system data:
        % ======================
        if strcmp(blockType, 'SubSystem') == 1
            subsys = find_system(sys, 'name', internalBlockName);
            if isobject(subsys) == 0 && isempty(subsys) == 0
          %      openFunc = get_param(subsys, 'OpenFcn');
          %      if isempty(openFunc)
                    try
                        subsys = subsys(1);
                        getSystemData(subsys, publisher, sysPath, helper, publishDir, path2mdl, withRefModels, moveToAction, sys);
                        open_system(sys);
                    catch e
                        disp(e);
                    end    
           %     end
            end
        end
        
        %get Reference Model data:
        % ======================
        if strcmp(blockType, 'ModelReference') == 1
            refMdlName = get_param(internalBlockPath, 'ModelName');
            refSysName = regexprep(refMdlName, '\n', ' ');
            refSysUriName = char(helper.legalizeMdlElemName(refSysName));
            publisher.addReferenceSystem(refSysName, refSysUriName);
            
            if strcmp(withRefModels, 'include') == 1 || strcmp(withRefModels, 'includeRecursive') == 1
                if dmConn.doesSimulinkModelExistOnDM(refSysName) == 1
                    msg = sprintf('Model "%s" already exist in DM, skiping...',refSysName); 
                    disp(msg);
                else    
                    if strcmp(moveToAction, 'publish') == 1
                        msg = sprintf('Publishing reference model "%s" to DM...',refSysName); 
                    else    
                        msg = sprintf('Moving reference model "%s" to DM...',refSysName); 
                    end    
                    disp(msg);
              
                    if strcmp(withRefModels, 'include') == 1
                        withRefModels = 'exclude';
                    end
                
                    bSuccess = uploadSimulinkModelToDM(refSysName, withRefModels, moveToAction, origSystem);
                    if bSuccess == 1
                        if strcmp(moveToAction, 'publish') == 1
                            disp('Publish reference model done');
                        else    
                            set_param(refSysName,'PostSaveFcn','postOpenFromDMActions()');
                            save_system(refSysName);
			
                            disp('Move reference model done.');
		
                            %last, lock the model since save/moveto release the locking
                            if strcmp(moveToAction, 'MoveToAndLock') == 1
                                dmConn.lockSimulinkModel(refSysName);
                            end
                        end    
                        close_system(refSysName);
                    else
                        if strcmp(moveToAction, 'publish') == 1
                            disp('Failed to publish reference model to DM');
                        else    
                            disp('Failed to move reference model to DM');
                        end
                    end
                end
            else
                % if ref model doesn't exist in the current Project Area
                % ask the user to provide the Project Area
                if dmConn.doesSimulinkModelExistOnDM(refSysName) == 0
                    refSysUri = char(dmConn.getSimulinkRefModelUri(refSysName));
                    if isempty(refSysUri) == 0
                        projectArea = char(dmConn.getProjectAreaOfSimulinkModel(refSysName, refSysUri));
                    else    
                        projectArea = char(dmConn.openSelectRefModelProjectAreaDialog(refSysName));
                        if isempty(projectArea) == 1
                            ex = MException();
                            throw(ex);
                        end   
                    end    
                    simBlock.setRefModelProjectArea(projectArea);
                end    
            end
        end
    end
end
    
function printSystemDiagram(sys, publishDir, helper, origSystem, simSystem)
    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    
    handle = get_param(sys, 'Handle');
    origModelHandle = get_param(origSystem, 'Handle');
    try
        % the model must be opened before saveas bitmap
        if handle ~= origModelHandle       
            open_system(handle);
        end
        
        % turn off selection before saving the diagram bitmap
        selectedHandle = find_system(sys, 'SearchDepth', 1, 'FindAll', 'on','Type', 'Block', 'Selected', 'on');
        if isempty(selectedHandle) == 0
            set_param(selectedHandle, 'Selected', 'off');
        end
         
        % make sure all elements fit in the diagram 
        zoomFactor = get_param(sys, 'ZoomFactor');
        set_param(sys, 'ZoomFactor', 'FitSystem');
        newZoomFactor = get_param(sys, 'ZoomFactor');
        simSystem.setZoomFactor(newZoomFactor);
        
        saveas(handle, diagPicDir);
        
        % restore selection
        if isempty(selectedHandle) == 0
            set_param(selectedHandle, 'Selected', 'on');
        end
        
        % restore old zoom
        set_param(sys, 'ZoomFactor', zoomFactor);
        
        % restore the old model if needed
       % if handle ~= origModelHandle  
       %     open_system(origModelHandle);
       % end
    catch e
        disp(e);
        close_system(handle);
        open_system(origModelHandle);
    end    
end
     
function printChartDiagram(sys, sysPath, publishDir, helper, origSystem, simSystem)
    orient(sys,'landscape');

    internalSysName = get_param(sys,'name');
    tempSysUriName = strrep(internalSysName, '/', '_');
    sysUriName = char(helper.legalizeMdlElemName(tempSysUriName));
    diagPicDir = strcat(publishDir,'/', sysUriName ,'.bmp');
    
    handle = get_param(sys, 'Handle');
    origModelHandle = get_param(origSystem, 'Handle');
    try
        % the model must be opened before saveas bitmap
        if handle ~= origModelHandle       
            open_system(handle);
        end
        
        % make sure all elements fit in the diagram 
        zoomFactor = get_param(sys, 'ZoomFactor');
        set_param(sys, 'ZoomFactor', 'FitSystem');
        newZoomFactor = get_param(sys, 'ZoomFactor');
        simSystem.setZoomFactor(newZoomFactor);
        
        sfprint(sysPath, 'bitmap', diagPicDir, 0);
     
        % restore old zoom
        set_param(sys, 'ZoomFactor', zoomFactor);
        
    catch e
        disp(e); 
        close_system(handle);
        open_system(origModelHandle);
    end    
end